/*!
 * \copyright Copyright (c) 2017-2023 Governikus GmbH & Co. KG, Germany
 */

#include "LocalWebSocketServer.h"

using namespace governikus;

LocalWebSocketServer::~LocalWebSocketServer() = default;
