varying highp vec2 qt_TexCoord0;
uniform highp float qt_Opacity;
uniform lowp sampler2D source;
uniform lowp sampler2D maskSource;

void main() {
	gl_FragColor = texture2D(source, qt_TexCoord0.st) * texture2D(maskSource, qt_TexCoord0.st).a * qt_Opacity;
}